#!/usr/bin/python2 -s

# Copyright 2010 by Carnegie Mellon University

# @OPENSOURCE_HEADER_START@
# Use of the Prism trend script and related source code is subject
# to the terms of the following licenses:
# 
# GNU Public License (GPL) Rights pursuant to Version 2, June 1991
# Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
# 
# NO WARRANTY
# 
# ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
# PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
# PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
# "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
# KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
# LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
# MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
# OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
# SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
# TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
# WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
# LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
# CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
# CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
# DELIVERABLES UNDER THIS LICENSE.
# 
# Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
# Mellon University, its trustees, officers, employees, and agents from 
# all claims or demands made against them (and any related losses, 
# expenses, or attorney's fees) arising out of, or relating to Licensee's 
# and/or its sub licensees' negligent use or willful misuse of or 
# negligent conduct or willful misconduct regarding the Software, 
# facilities, or other rights or assistance granted by Carnegie Mellon 
# University under this License, including, but not limited to, any 
# claims of product liability, personal injury, death, damage to 
# property, or violation of any laws or regulations.
# 
# Carnegie Mellon University Software Engineering Institute authored 
# documents are sponsored by the U.S. Department of Defense under 
# Contract FA8721-05-C-0003. Carnegie Mellon University retains 
# copyrights in all material produced under this contract. The U.S. 
# Government retains a non-exclusive, royalty-free license to publish or 
# reproduce these documents, or allow others to do so, for U.S. 
# Government purposes only pursuant to the copyright license under the 
# contract clause at 252.227.7013.
# @OPENSOURCE_HEADER_END@

from netsa import script
from netsa.times import bin_datetime, silk_hour
from netsa.tools.service import check_pidfile
from netsa.util.shell import run_parallel, command

from datetime import datetime, timedelta
import sys

script.set_title("Flow Trends Cron Job")
script.set_description("""
    Ensure a trend database is loaded with recent data.
""")
script.set_contact("NetSA Help <netsa-help@cert.org>")
script.add_author("J. Prevost <prevost1@cert.org>")

script.add_path_param(
    "config-path", "Read trend configuration from this file", required=True)
script.add_int_param(
    "load-days", "Load the N most recent days of data", default=14)
script.add_int_param(
    "reload-hours", "Force load the N most recent hours of data", default=2)
script.add_path_param(
    "pidfile-path", "Use the specified PID file to prevent simultaneous runs",
    default="/var/run/trend-cron.pid")
script.add_path_param(
    "trend-path", "Trend script executable", default="trend", expert=True)

def main():
    config_path = script.get_param('config-path')
    pidfile_path = script.get_param('pidfile-path')
    load_days = script.get_param('load-days')
    reload_hours = script.get_param('reload-hours')
    trend_path = script.get_param('trend-path')    
    if not check_pidfile(pidfile_path):
        # Another copy is already running
        sys.exit(0)
    current_hour = bin_datetime(timedelta(hours=1), datetime.utcnow())
    # Load the last two hours no matter what
    start_hour = silk_hour(current_hour - timedelta(hours=reload_hours+1))
    end_hour = silk_hour(current_hour - timedelta(hours=1))
    run_parallel(command(trend_path,
                         "--start-date=" + start_hour,
                         "--end-date=" + end_hour,
                         "--config-path=" + config_path,
                         "--load-database"))
    # Load any missing data from the entire time period
    start_hour = silk_hour(current_hour - timedelta(days=load_days))
    end_hour = silk_hour(current_hour - timedelta(hours=1))
    run_parallel(command(trend_path,
                         "--start-date=" + start_hour,
                         "--end-date=" + end_hour,
                         "--config-path=" + config_path,
                         "--auto-load-database"))

script.execute(main)
